@@ -4,7 +4,10 @@ import requests
 from urecord import Record
 from urlobject import URLObject, encode_component
 
+from conneg.accept import make_accept_header
+from conneg.default import DEFAULT_REGISTER
 from http import requests_http_backend
+from utils.caseless_dict import CaselessDictionary
 
 __all__ = ['API']
 
@@ -81,6 +84,49 @@ class API(Record('url', 'http')):
 
         return unicode(self) + other
 
+    @replaces_url
+    def __and__(self, params):
+
+        """
+        Add parameters to this URL.
+
+        Accepts pairs or dictionaries:
+
+            >>> API(u'http://example.com/') & ('a', 'b')
+            <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/') & {'a': 'b'}
+            <intessa.API(u'http://example.com/?a=b')>
+
+        Multiple additions of the same parameter will keep all of them, in the
+        order they were specified:
+
+            >>> API(u'http://example.com/') & ('a', 'b') & ('a', 'c')
+            <intessa.API(u'http://example.com/?a=b&a=c')>
+        """
+
+        return self.url & params
+
+    @replaces_url
+    def __or__(self, params):
+
+        """
+        Add parameters to this URL, replacing existing parameters.
+
+        Accepts pairs or dictionaries:
+
+            >>> API(u'http://example.com/') | ('a', 'b')
+            <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/') | {'a': 'b'}
+            <intessa.API(u'http://example.com/?a=b')>
+
+        Multiple additions of the same parameter will replace old values:
+
+            >>> API(u'http://example.com/') | ('a', 'b') | ('a', 'c')
+            <intessa.API(u'http://example.com/?a=c')>
+        """
+
+        return self.url | params
+
     @replaces_url
     def __getattr__(self, attr):
 
@@ -207,11 +253,75 @@ class API(Record('url', 'http')):
         return self.url.parent() / (base + file_ext)
 
     def __call__(self, *args, **kwargs):
-        """Perform an HTTP request and return a decoded response."""
+
+        """
+        Perform an HTTP request and return a :class:`~intessa.response.Response`.
+
+        :param data:
+            A Python object representing the body of the request.
+
+            There are several valid values this parameter can take:
+
+            *   A bytestring, with a ``Content-Type`` header (given in
+                ``headers``).
+            *   A file-like object, with ``Content-Type`` and
+                ``Content-Length`` headers.
+            *   Any other Python object, with the ``type`` parameter provided.
+
+            In the first two instances, it will be sent as-is. In the second,
+            it will first be encoded to a bytestring, using the codec for the
+            specified type.
+
+        :param type:
+            The Internet media type with which the ``data`` argument should be
+            encoded. This will be used as a lookup type against the
+            ``codec_register``.
+
+        :param codec_register:
+            A :class:`~intessa.conneg.codec_base.CodecRegister` to use for
+            encoding the request body and decoding the response. Defaults to
+            :data:`~intessa.conneg.default.DEFAULT_REGISTER`.
+
+        :param accept:
+            A specification of acceptable response types, which will be turned
+            into an HTTP ``Accept`` header.
+
+            This argument can be a string representing a media type (see:
+            :class:`~intessa.conneg.content_type.ContentType`), a list of such
+            strings, or a list of ``(media_type, quality)`` pairs, where
+            ``quality`` is a floating-point number. See the documentation for
+            :func:`~intessa.conneg.accept.make_accept_header` for examples of
+            how this argument is turned into an ``Accept`` header.
+
+        :param headers:
+            A dictionary of raw HTTP headers to send.
+        """
 
         method = kwargs.pop('method', 'GET')
-        return self.http(self, method, self.url, *args, **kwargs)
+        accept = kwargs.pop('accept', None)
+        headers = CaselessDictionary(kwargs.pop('headers', {}))
 
+        if accept is not None:
+            headers['Accept'] = make_accept_header(accept)
+
+        if 'data' in kwargs:
+            data = kwargs.pop('data')
+            if method == 'GET':
+                method = 'POST'
+            if ((isinstance(data, str) and 'content-type' in headers) or
+                (hasattr(data, 'read') and
+                 'content-type' in headers and
+                 'content-length' in headers)):
+                kwargs['data'] = data
+            elif 'type' not in kwargs:
+                raise TypeError("Request data provided with no content type information")
+            else:
+                codec_register = kwargs.pop('codec_register', DEFAULT_REGISTER)
+                headers['content-type'], kwargs['data'] = codec_register.encode(
+                    kwargs.pop('type'), data)
+
+        kwargs['headers'] = headers
+        return self.http(self, method, self.url, *args, **kwargs)
 
     def _post(self, *args, **kwargs):
         """Shim for issuing POST requests."""
